==> org.opennms.features.flows.classification.engine.impl-25.1.0-sources.jar: org/opennms/netmgt/flows/classification/internal/DefaultClassificationService.java <==
/*******************************************************************************
 * This file is part of OpenNMS(R).
 *
 * Copyright (C) 2018-2018 The OpenNMS Group, Inc.
 * OpenNMS(R) is Copyright (C) 1999-2018 The OpenNMS Group, Inc.
 *
 * OpenNMS(R) is a registered trademark of The OpenNMS Group, Inc.
 *
 * OpenNMS(R) is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published
 * by the Free Software Foundation, either version 3 of the License,
 * or (at your option) any later version.
 *
 * OpenNMS(R) is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with OpenNMS(R).  If not, see:
 *      http://www.gnu.org/licenses/
 *
 * For more information contact:
 *     OpenNMS(R) Licensing <license@opennms.org>
 *     http://www.opennms.org/
 *     http://www.opennms.com/
 *******************************************************************************/

package org.opennms.netmgt.flows.classification.internal;

import java.io.InputStream;
import java.util.List;
import java.util.NoSuchElementException;
import java.util.Objects;

import org.opennms.core.criteria.Criteria;
import org.opennms.core.criteria.CriteriaBuilder;
import org.opennms.netmgt.flows.classification.ClassificationEngine;
import org.opennms.netmgt.flows.classification.ClassificationRequest;
import org.opennms.netmgt.flows.classification.ClassificationService;
import org.opennms.netmgt.flows.classification.FilterService;
import org.opennms.netmgt.flows.classification.csv.CsvImportResult;
import org.opennms.netmgt.flows.classification.csv.CsvService;
import org.opennms.netmgt.flows.classification.error.ErrorContext;
import org.opennms.netmgt.flows.classification.error.Errors;
import org.opennms.netmgt.flows.classification.exception.CSVImportException;
import org.opennms.netmgt.flows.classification.exception.ClassificationException;
import org.opennms.netmgt.flows.classification.exception.InvalidRuleException;
import org.opennms.netmgt.flows.classification.internal.csv.CsvServiceImpl;
import org.opennms.netmgt.flows.classification.internal.validation.GroupValidator;
import org.opennms.netmgt.flows.classification.internal.validation.RuleValidator;
import org.opennms.netmgt.flows.classification.persistence.api.ClassificationGroupDao;
import org.opennms.netmgt.flows.classification.persistence.api.ClassificationRuleDao;
import org.opennms.netmgt.flows.classification.persistence.api.Group;
import org.opennms.netmgt.flows.classification.persistence.api.Rule;
import org.springframework.transaction.support.TransactionCallback;
import org.springframework.transaction.support.TransactionOperations;

public class DefaultClassificationService implements ClassificationService {

    private final ClassificationRuleDao classificationRuleDao;

    private final ClassificationGroupDao classificationGroupDao;

    private final ClassificationEngine classificationEngine;

    private final RuleValidator ruleValidator;

    private final CsvService csvService;

    private final GroupValidator groupValidator;

    private final TransactionOperations transactionTemplate;

    public DefaultClassificationService(ClassificationRuleDao classificationRuleDao,
                                        ClassificationGroupDao classificationGroupDao,
                                        ClassificationEngine classificationEngine,
                                        FilterService filterService,
                                        TransactionOperations transactionOperations) {
        this.classificationRuleDao = Objects.requireNonNull(classificationRuleDao);
        this.classificationGroupDao = Objects.requireNonNull(classificationGroupDao);
        this.classificationEngine = Objects.requireNonNull(classificationEngine);
        this.transactionTemplate = Objects.requireNonNull(transactionOperations);
        this.ruleValidator = new RuleValidator(filterService);
        this.groupValidator = new GroupValidator(classificationRuleDao);
        this.csvService = new CsvServiceImpl(ruleValidator);
    }

    @Override
    public List<Rule> findMatchingRules(Criteria criteria) {
        return classificationRuleDao.findMatching(criteria);
    }

    @Override
    public int countMatchingRules(Criteria criteria) {
        return classificationRuleDao.countMatching(criteria);
    }

    @Override
    public Rule getRule(int ruleId) {
        Rule rule = classificationRuleDao.get(ruleId);
        if (rule == null) throw new NoSuchElementException();
        return rule;
    }

    @Override
    public Integer saveRule(Rule rule) throws InvalidRuleException {
        return runInTransaction((status) -> {

            ruleValidator.validate(rule);

            final Group group = classificationGroupDao.get(rule.getGroup().getId());
            if(group == null) {
                throw new NoSuchElementException(String.format("Unknown group with id=%s", rule.getGroup().getId()));
            }
            assertRuleIsNotInReadOnlyGroup(group);
            groupValidator.validate(group, rule);

            // persist
            group.addRule(rule);
            final Integer ruleId = classificationRuleDao.save(rule);
            updateRulePositionsAndReloadEngine(PositionUtil.sortRulePositions(rule));

            return ruleId;
        });
    }

    @Override
    public void importRules(int groupId, InputStream inputStream, boolean hasHeader, boolean deleteExistingRules) throws CSVImportException {
        runInTransaction(status -> {

            // Get and check group
            Group group = classificationGroupDao.get(groupId);
            if(group == null) {
                throw new ClassificationException(ErrorContext.Name, Errors.GROUP_NOT_FOUND, groupId);
            }
            if(group.isReadOnly()) {
                throw new ClassificationException(ErrorContext.Name, Errors.GROUP_READ_ONLY, groupId);
            }

            // Parse and validate the rules
            final CsvImportResult result = csvService.parseCSV(group, inputStream, hasHeader);
            if (!result.isSuccess()) {
                throw new CSVImportException(result);
            }

            // Remove existing rules and afterwards add new rules
            if (deleteExistingRules) {
                for (Rule eachRule : group.getRules()) {
                    classificationRuleDao.delete(eachRule);
                }
                group.getRules().clear();
            }

            // Add new rules
            final List<Rule> rules = result.getRules();
            for (int i=0; i<rules.size(); i++) {
                final Rule rule = rules.get(i);
                try {
                    groupValidator.validate(group, rule);
                    group.addRule(rule);
                } catch (ClassificationException ex) {
                    result.markError(i, ex.getError());
                }
            }

            // before continuing, verify everything is okay, otherwise bail
            if (!result.isSuccess()) {
                throw new CSVImportException(result);
            }

            updateRulePositionsAndReloadEngine(PositionUtil.sortRulePositions(group.getRules()));
            classificationGroupDao.saveOrUpdate(group);

            return null;
        });
    }

    @Override
    public String exportRules(int groupId) {
        final Group group = classificationGroupDao.get(groupId);
        if (group == null) throw new NoSuchElementException();
        final String csvContent = csvService.createCSV(group.getRules());
        return csvContent;
    }

    @Override
    public void deleteRules(int groupId) {
        CriteriaBuilder criteriaBuilder = new CriteriaBuilder(Rule.class).alias("group", "group");
        criteriaBuilder.eq("group.id", groupId);
        runInTransaction(status -> {
            Group group = classificationGroupDao.get(groupId);
            if (group == null) throw new NoSuchElementException();
            if(group.isReadOnly()) {
                throw new ClassificationException(ErrorContext.Entity, Errors.GROUP_READ_ONLY);
            }
            final Criteria criteria = criteriaBuilder.toCriteria();
            classificationRuleDao.findMatching(criteria).forEach(r -> classificationRuleDao.delete(r));
            classificationEngine.reload();
            return null;
        });
    }

    @Override
    public void deleteRule(int ruleId) {
        runInTransaction(status -> {
            final Rule rule = classificationRuleDao.get(ruleId);
            if (rule == null) throw new NoSuchElementException();
            assertRuleIsNotInReadOnlyGroup(rule);
            return runInTransaction(transactionStatus -> {
                // Remove from group, as it would be saved later otherwise
                final Group group = rule.getGroup();
                group.removeRule(rule);
                classificationRuleDao.delete(rule);
                updateRulePositionsAndReloadEngine(PositionUtil.sortRulePositions(group.getRules()));
                return null;
            });
        });
    }

    @Override
    public void updateRule(final Rule rule) {
        if (rule.getId() == null) throw new NoSuchElementException();
        assertRuleIsNotInReadOnlyGroup(rule);

        // Persist
        runInTransaction(status -> {
            ruleValidator.validate(rule);
            groupValidator.validate(rule.getGroup(), rule);
            classificationRuleDao.saveOrUpdate(rule);
            // reload to get updated group since we might just have switched groups
            Rule reloaded = classificationRuleDao.get(rule.getId());
            updateRulePositionsAndReloadEngine(PositionUtil.sortRulePositions(reloaded));
            return null;
        });
    }

    private void assertRuleIsNotInReadOnlyGroup(Rule rule) {
        assertRuleIsNotInReadOnlyGroup(rule.getGroup());
    }

    private void assertRuleIsNotInReadOnlyGroup(Group group) {
        if(group.isReadOnly()) {
            throw new ClassificationException(ErrorContext.Entity, Errors.GROUP_READ_ONLY);
        }
    }

    @Override
    public String classify(ClassificationRequest classificationRequest) {
        final String classification = classificationEngine.classify(classificationRequest);
        return classification;
    }

    @Override
    public List<Group> findMatchingGroups(Criteria criteria) {
        return classificationGroupDao.findMatching(criteria);
    }

    @Override
    public int countMatchingGroups(Criteria criteria) {
        return classificationGroupDao.countMatching(criteria);
    }

    @Override
    public Group getGroup(int groupId) {
        final Group group = classificationGroupDao.get(groupId);
        if (group == null) throw new NoSuchElementException();
        return group;
    }

    @Override
    public Integer saveGroup(Group group) {
        checkForDuplicateName(group);
        return runInTransaction((status) -> {
            Integer groupId = classificationGroupDao.save(group);
            updateGroupPositionsAndReloadEngine(PositionUtil.sortGroupPositions(group, classificationGroupDao.findAll()));
            return groupId;
        });
    }

    @Override
    public void deleteGroup(int groupId) {
        runInTransaction(status -> {
            final Group group = classificationGroupDao.get(groupId);
            if (group == null) {
                throw new NoSuchElementException();
            } else if(group.isReadOnly()) {
                throw new ClassificationException(ErrorContext.Entity, Errors.GROUP_READ_ONLY);
            }
            classificationGroupDao.delete(group);
            updateGroupPositionsAndReloadEngine(PositionUtil.sortGroupPositions(classificationGroupDao.findAll()));
            classificationEngine.reload();
            return null;
        });
    }

    @Override
    public void updateGroup(Group group) {
        if (group.getId() == null) throw new NoSuchElementException();
        checkForDuplicateName(group);
        runInTransaction(status -> {
            classificationGroupDao.saveOrUpdate(group);
            updateGroupPositionsAndReloadEngine(PositionUtil.sortGroupPositions(group, classificationGroupDao.findAll()));
            updateRulePositionsAndReloadEngine(PositionUtil.sortRulePositions(group.getRules()));
            return null;
        });
    }

    private <T> T runInTransaction(TransactionCallback<T> callback) {
        return transactionTemplate.execute(callback);
    }

    private void updateRulePositionsAndReloadEngine(final List<Rule> rules) {

        // Update position field
        for (int i=0; i<rules.size(); i++) {
            Rule rule = rules.get(i);
            rule.setPosition(i);
            classificationRuleDao.saveOrUpdate(rule);
        }

        // Reload engine
        classificationEngine.reload();
    }

    private void updateGroupPositionsAndReloadEngine(final List<Group> groups) {

        // Update position field
        for (int i=0; i<groups.size(); i++) {
            Group group = groups.get(i);
            group.setPosition(i);
            classificationGroupDao.saveOrUpdate(group);
        }

        // Reload engine
        classificationEngine.reload();
    }

    private void checkForDuplicateName (Group group) {
        CriteriaBuilder builder = new CriteriaBuilder(Group.class)
                .eq("name", group.getName());
        if(group.getId() !=null) {
            builder.ne("id", group.getId());
        }
        if(countMatchingGroups(builder.toCriteria()) > 0) {
            throw new ClassificationException(ErrorContext.Entity, Errors.GROUP_NAME_NOT_UNIQUE, group.getName());
        }
    }
}

==> org.opennms.features.flows.classification.engine.impl-25.1.2-sources.jar: org/opennms/netmgt/flows/classification/internal/DefaultClassificationService.java <==
/*******************************************************************************
 * This file is part of OpenNMS(R).
 *
 * Copyright (C) 2018-2018 The OpenNMS Group, Inc.
 * OpenNMS(R) is Copyright (C) 1999-2018 The OpenNMS Group, Inc.
 *
 * OpenNMS(R) is a registered trademark of The OpenNMS Group, Inc.
 *
 * OpenNMS(R) is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published
 * by the Free Software Foundation, either version 3 of the License,
 * or (at your option) any later version.
 *
 * OpenNMS(R) is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with OpenNMS(R).  If not, see:
 *      http://www.gnu.org/licenses/
 *
 * For more information contact:
 *     OpenNMS(R) Licensing <license@opennms.org>
 *     http://www.opennms.org/
 *     http://www.opennms.com/
 *******************************************************************************/

package org.opennms.netmgt.flows.classification.internal;

import java.io.InputStream;
import java.util.List;
import java.util.NoSuchElementException;
import java.util.Objects;
import java.util.function.Supplier;

import org.opennms.core.criteria.Criteria;
import org.opennms.core.criteria.CriteriaBuilder;
import org.opennms.netmgt.dao.api.SessionUtils;
import org.opennms.netmgt.flows.classification.ClassificationEngine;
import org.opennms.netmgt.flows.classification.ClassificationRequest;
import org.opennms.netmgt.flows.classification.ClassificationService;
import org.opennms.netmgt.flows.classification.FilterService;
import org.opennms.netmgt.flows.classification.csv.CsvImportResult;
import org.opennms.netmgt.flows.classification.csv.CsvService;
import org.opennms.netmgt.flows.classification.error.ErrorContext;
import org.opennms.netmgt.flows.classification.error.Errors;
import org.opennms.netmgt.flows.classification.exception.CSVImportException;
import org.opennms.netmgt.flows.classification.exception.ClassificationException;
import org.opennms.netmgt.flows.classification.exception.InvalidRuleException;
import org.opennms.netmgt.flows.classification.internal.csv.CsvServiceImpl;
import org.opennms.netmgt.flows.classification.internal.validation.GroupValidator;
import org.opennms.netmgt.flows.classification.internal.validation.RuleValidator;
import org.opennms.netmgt.flows.classification.persistence.api.ClassificationGroupDao;
import org.opennms.netmgt.flows.classification.persistence.api.ClassificationRuleDao;
import org.opennms.netmgt.flows.classification.persistence.api.Group;
import org.opennms.netmgt.flows.classification.persistence.api.Rule;

public class DefaultClassificationService implements ClassificationService {

    private final ClassificationRuleDao classificationRuleDao;

    private final ClassificationGroupDao classificationGroupDao;

    private final ClassificationEngine classificationEngine;

    private final RuleValidator ruleValidator;

    private final CsvService csvService;

    private final GroupValidator groupValidator;

    private final SessionUtils sessionUtils;

    public DefaultClassificationService(ClassificationRuleDao classificationRuleDao,
                                        ClassificationGroupDao classificationGroupDao,
                                        ClassificationEngine classificationEngine,
                                        FilterService filterService,
                                        SessionUtils sessionUtils) {
        this.classificationRuleDao = Objects.requireNonNull(classificationRuleDao);
        this.classificationGroupDao = Objects.requireNonNull(classificationGroupDao);
        this.classificationEngine = Objects.requireNonNull(classificationEngine);
        this.sessionUtils = Objects.requireNonNull(sessionUtils);
        this.ruleValidator = new RuleValidator(filterService);
        this.groupValidator = new GroupValidator(classificationRuleDao);
        this.csvService = new CsvServiceImpl(ruleValidator);
    }

    @Override
    public List<Rule> findMatchingRules(Criteria criteria) {
        return classificationRuleDao.findMatching(criteria);
    }

    @Override
    public int countMatchingRules(Criteria criteria) {
        return classificationRuleDao.countMatching(criteria);
    }

    @Override
    public Rule getRule(int ruleId) {
        Rule rule = classificationRuleDao.get(ruleId);
        if (rule == null) throw new NoSuchElementException();
        return rule;
    }

    @Override
    public Integer saveRule(Rule rule) throws InvalidRuleException {
        return runInTransaction(() -> {

            ruleValidator.validate(rule);

            final Group group = classificationGroupDao.get(rule.getGroup().getId());
            if(group == null) {
                throw new NoSuchElementException(String.format("Unknown group with id=%s", rule.getGroup().getId()));
            }
            assertRuleIsNotInReadOnlyGroup(group);
            groupValidator.validate(group, rule);

            // persist
            group.addRule(rule);
            final Integer ruleId = classificationRuleDao.save(rule);
            updateRulePositionsAndReloadEngine(PositionUtil.sortRulePositions(rule));

            return ruleId;
        });
    }

    @Override
    public void importRules(int groupId, InputStream inputStream, boolean hasHeader, boolean deleteExistingRules) throws CSVImportException {
        runInTransaction(() -> {

            // Get and check group
            Group group = classificationGroupDao.get(groupId);
            if(group == null) {
                throw new ClassificationException(ErrorContext.Name, Errors.GROUP_NOT_FOUND, groupId);
            }
            if(group.isReadOnly()) {
                throw new ClassificationException(ErrorContext.Name, Errors.GROUP_READ_ONLY, groupId);
            }

            // Parse and validate the rules
            final CsvImportResult result = csvService.parseCSV(group, inputStream, hasHeader);
            if (!result.isSuccess()) {
                throw new CSVImportException(result);
            }

            // Remove existing rules and afterwards add new rules
            if (deleteExistingRules) {
                for (Rule eachRule : group.getRules()) {
                    classificationRuleDao.delete(eachRule);
                }
                group.getRules().clear();
            }

            // Add new rules
            final List<Rule> rules = result.getRules();
            for (int i=0; i<rules.size(); i++) {
                final Rule rule = rules.get(i);
                try {
                    groupValidator.validate(group, rule);
                    group.addRule(rule);
                } catch (ClassificationException ex) {
                    result.markError(i, ex.getError());
                }
            }

            // before continuing, verify everything is okay, otherwise bail
            if (!result.isSuccess()) {
                throw new CSVImportException(result);
            }

            updateRulePositionsAndReloadEngine(PositionUtil.sortRulePositions(group.getRules()));
            classificationGroupDao.saveOrUpdate(group);

            return null;
        });
    }

    @Override
    public String exportRules(int groupId) {
        final Group group = classificationGroupDao.get(groupId);
        if (group == null) throw new NoSuchElementException();
        final String csvContent = csvService.createCSV(group.getRules());
        return csvContent;
    }

    @Override
    public void deleteRules(int groupId) {
        CriteriaBuilder criteriaBuilder = new CriteriaBuilder(Rule.class).alias("group", "group");
        criteriaBuilder.eq("group.id", groupId);
        runInTransaction(() -> {
            Group group = classificationGroupDao.get(groupId);
            if (group == null) throw new NoSuchElementException();
            if(group.isReadOnly()) {
                throw new ClassificationException(ErrorContext.Entity, Errors.GROUP_READ_ONLY);
            }
            final Criteria criteria = criteriaBuilder.toCriteria();
            classificationRuleDao.findMatching(criteria).forEach(r -> classificationRuleDao.delete(r));
            classificationEngine.reload();
            return null;
        });
    }

    @Override
    public void deleteRule(int ruleId) {
        runInTransaction(() -> {
            final Rule rule = classificationRuleDao.get(ruleId);
            if (rule == null) throw new NoSuchElementException();
            assertRuleIsNotInReadOnlyGroup(rule);
            return runInTransaction(() -> {
                // Remove from group, as it would be saved later otherwise
                final Group group = rule.getGroup();
                group.removeRule(rule);
                classificationRuleDao.delete(rule);
                updateRulePositionsAndReloadEngine(PositionUtil.sortRulePositions(group.getRules()));
                return null;
            });
        });
    }

    @Override
    public void updateRule(final Rule rule) {
        if (rule.getId() == null) throw new NoSuchElementException();
        assertRuleIsNotInReadOnlyGroup(rule);

        // Persist
        runInTransaction(() -> {
            ruleValidator.validate(rule);
            groupValidator.validate(rule.getGroup(), rule);
            classificationRuleDao.saveOrUpdate(rule);
            // reload to get updated group since we might just have switched groups
            Rule reloaded = classificationRuleDao.get(rule.getId());
            updateRulePositionsAndReloadEngine(PositionUtil.sortRulePositions(reloaded));
            return null;
        });
    }

    private void assertRuleIsNotInReadOnlyGroup(Rule rule) {
        assertRuleIsNotInReadOnlyGroup(rule.getGroup());
    }

    private void assertRuleIsNotInReadOnlyGroup(Group group) {
        if(group.isReadOnly()) {
            throw new ClassificationException(ErrorContext.Entity, Errors.GROUP_READ_ONLY);
        }
    }

    @Override
    public String classify(ClassificationRequest classificationRequest) {
        final String classification = classificationEngine.classify(classificationRequest);
        return classification;
    }

    @Override
    public List<Group> findMatchingGroups(Criteria criteria) {
        return classificationGroupDao.findMatching(criteria);
    }

    @Override
    public int countMatchingGroups(Criteria criteria) {
        return classificationGroupDao.countMatching(criteria);
    }

    @Override
    public Group getGroup(int groupId) {
        final Group group = classificationGroupDao.get(groupId);
        if (group == null) throw new NoSuchElementException();
        return group;
    }

    @Override
    public Integer saveGroup(Group group) {
        checkForDuplicateName(group);
        return runInTransaction(() -> {
            Integer groupId = classificationGroupDao.save(group);
            updateGroupPositionsAndReloadEngine(PositionUtil.sortGroupPositions(group, classificationGroupDao.findAll()));
            return groupId;
        });
    }

    @Override
    public void deleteGroup(int groupId) {
        runInTransaction(() -> {
            final Group group = classificationGroupDao.get(groupId);
            if (group == null) {
                throw new NoSuchElementException();
            } else if(group.isReadOnly()) {
                throw new ClassificationException(ErrorContext.Entity, Errors.GROUP_READ_ONLY);
            }
            classificationGroupDao.delete(group);
            updateGroupPositionsAndReloadEngine(PositionUtil.sortGroupPositions(classificationGroupDao.findAll()));
            classificationEngine.reload();
            return null;
        });
    }

    @Override
    public void updateGroup(Group group) {
        if (group.getId() == null) throw new NoSuchElementException();
        checkForDuplicateName(group);
        runInTransaction(() -> {
            classificationGroupDao.saveOrUpdate(group);
            updateGroupPositionsAndReloadEngine(PositionUtil.sortGroupPositions(group, classificationGroupDao.findAll()));
            updateRulePositionsAndReloadEngine(PositionUtil.sortRulePositions(group.getRules()));
            return null;
        });
    }

    private <T> T runInTransaction(Supplier<T> supplier) {
        Objects.requireNonNull(supplier);
        return sessionUtils.withTransaction(supplier);
    }

    private void updateRulePositionsAndReloadEngine(final List<Rule> rules) {

        // Update position field
        for (int i=0; i<rules.size(); i++) {
            Rule rule = rules.get(i);
            rule.setPosition(i);
            classificationRuleDao.saveOrUpdate(rule);
        }

        // Reload engine
        classificationEngine.reload();
    }

    private void updateGroupPositionsAndReloadEngine(final List<Group> groups) {

        // Update position field
        for (int i=0; i<groups.size(); i++) {
            Group group = groups.get(i);
            group.setPosition(i);
            classificationGroupDao.saveOrUpdate(group);
        }

        // Reload engine
        classificationEngine.reload();
    }

    private void checkForDuplicateName (Group group) {
        CriteriaBuilder builder = new CriteriaBuilder(Group.class)
                .eq("name", group.getName());
        if(group.getId() !=null) {
            builder.ne("id", group.getId());
        }
        if(countMatchingGroups(builder.toCriteria()) > 0) {
            throw new ClassificationException(ErrorContext.Entity, Errors.GROUP_NAME_NOT_UNIQUE, group.getName());
        }
    }
}
